#!/usr/bin/env python
#
# MOMAP toolkit, http://momap.net.cn
# By Deping Hu, hudp@mail.tsinghua.edu.cn, Tsinghua University
#
# Copyright (2018) Zhigang Shuai Group, Tsinghua University.

import sys
import re

# Usage:
#       python dalton_get_info.py dalton.out
#    or
#       dalton_get_info.py dalton.out
#
# It will collect data from the dalton output file, calculate the modulus
# or average of EDMA, EDME, or Hso.
# 

dalton_out_filename=sys.argv[1]

dalton_out=open(dalton_out_filename,'r')

line = "NOT EMPTY LINE"

switch=0

# Note: Start from version 2017, the access index should be -4, otherwise -2
while line != "":
   pattern = re.compile(r'Spin-orbit coupling constant \(X1SPNSCA\)')

   line = dalton_out.readline()
   m = pattern.search(line)
   if m is not None:
       switch = 1
       record = line.split()
       spin_x = float(record[-4])
       print ('Hso_x:   %12g cm-1' % spin_x)
       break

while line != "":
   pattern = re.compile(r'Spin-orbit coupling constant \(Y1SPNSCA\)')

   line = dalton_out.readline()
   m = pattern.search(line)
   if m is not None:
       record = line.split()
       spin_y = float(record[-4])
       print ('Hso_y:   %12g cm-1' % spin_y)
       break

while line != "":
   pattern = re.compile(r'Spin-orbit coupling constant \(Z1SPNSCA\)')

   line = dalton_out.readline()
   m = pattern.search(line)
   if m is not None:
       record = line.split()
       spin_z = float(record[-4])
       print ('Hso_z:   %12g cm-1' % spin_z)
       break


if switch ==1:
   spin_ave = ((spin_x**2+spin_y**2+spin_z**2)/3)**0.5
   print ('Hso_ave: %12g cm-1' % spin_ave)


dalton_out.seek(0)
line = dalton_out.readline()

while line != "":

   pattern = re.compile(r'Partial rates \(ECSO\): X-polarization')

   line = dalton_out.readline()
   m = pattern.search(line)
   if m is not None:
       record = line.split()
       dipole_x = float(record[-1])
       print ('dipole_sub_state_x: %12g au' % dipole_x)
       break

while line != "":

   pattern = re.compile(r'Partial rates \(ECSO\): Y-polarization')

   line = dalton_out.readline()
   m = pattern.search(line)
   if m is not None:
       switch = 2
       record = line.split()
       dipole_y = float(record[-1])
       print ('dipole_sub_state_y: %12g au' % dipole_y)
       break

while line != "":

   pattern = re.compile(r'Partial rates \(ECSO\): Z-polarization')

   line = dalton_out.readline()
   m = pattern.search(line)
   if m is not None:
       record = line.split()
       dipole_z = float(record[-1])
       print ('dipole_sub_state_z: %12g au' % dipole_z)
       break

if switch ==2:
   dipole_ave = (dipole_x**2+dipole_y**2+dipole_z**2)**0.5 * 2.5417
   print ('dipole_average(mod): %12g debye' % dipole_ave)


if switch ==0:
   print ('the dalton job is not completed')

dalton_out.close()
