#!/bin/sh

cat <<%EOF%
#############################################################
#                                                           #
#                           Dipole                          #
#                         Yingli Niu                        #
#                         2008-09-09                        #
#                                                           #
#############################################################
%EOF%

##########################################################
function StartJob(){
  if [ -f finished ] 
  then
    echo
    echo "The job has been done!"
    echo
    exit 0
  fi
  ##########################################################
  WORKDIR=`pwd`
  DPLDIR=$WORKDIR/dipole
  MAXJOBS=1
  mfile=""

  tmpfile=temp
  tmpfile2=temp2

  # take care of decimal points
  LANG="C"
  LC_ALL="C"
  LC_CTYPE="C"
  remoteshell=ssh

  HOST=`hostname`

  echo
  echo " running on $HOST"
  echo " date: "`date '+Time: %Y-%m-%d %T'`
  echo
}
############################################################

############################################################
function CheckPara() {
  if [ -f $mfile ]; then
    nmach=`cat $mfile | wc -l`
    # copy list of hosts to RAWHOSTLIST and remove domain-suffix
    RAWHOSTLIST=`cut -f 1 -d "." $mfile`
    # give all hosts in the list a unique name
    HOSTLIST=""
    for machine in $RAWHOSTLIST;
    do
      i=0
      unique_name=0
      while [ $unique_name -eq 0 ];
      do
        i=`expr $i + 1`
        try=$machine#$i
        unique_name=255
        for machine2 in $HOSTLIST;
        do
          if [ $machine2 == $try ];
          then
            unique_name=0
          fi
        done
      done
      HOSTLIST="$HOSTLIST $try"
    done
   
    # test existence of hosts
    for machine in $HOSTLIST ; do
      loginname=`echo $machine | cut -f 1 -d "#"`
      if [ $loginname != $HOST ]; then
        $remoteshell $loginname "echo login-TEST on $machine"
        echo $loginname $?
        if [ $? -ne 0 ]; then
          echo 
          echo "Error logging in on $machine"
          echo "Check your machine-file $machfile"
          exit 1
        fi
      else
        echo $machine is local host
      fi
    done
  else
    echo; echo "Can't find machine-file $mfile"
    exit 1;
  fi
}
############################################################

############################################################
function DoDipole() {
  cd $WORKDIR
  dfile1=`grep dfile $ifile | cut -d "=" -f 2`
  nnn=`echo $dfile1 | grep "\"" | wc -l`
  if [ $nnn = "1" ]; then
    dfile=`echo $dfile1 | cut -d "\"" -f 2`
  fi
  rm -rf $dfile1
  cd $DPLDIR
  NDir=`grep "Number of files" $dfile | cut -d ":" -f 2`
  dirs=`cat $dfile | grep -v ":" `

  i=1
  for dir in $dirs; do

    [ -f stop ] && echo "FOUND stop FILE. TERMINATING ... " && exit 0
   
    if [ -s $dir.log ]; then
      echo "step $dir was already done (# $i of $NDir)"
    else
      echo "step $dir started (# $i of $NDir)"
      # non-parallel runs
      if [ $MAXJOBS -eq 1 ]; then
        machine=$HOSTNAME
        if [ ! -f "lockhost.$machine" ] && [ ! -f "lock.$dir" ]; then
          $scriptdir/rsingle.sh $dir $machine $WORKDIR $DPLDIR $scratch
        fi
      else
        # look for a free host
        for machine in $HOSTLIST; do
          if [ ! -f "lockhost.$machine" ] && [ ! -f "lock.$dir" ]; then
            touch lock.$dir
            touch lockhost.$machine
            loginname=`echo $machine | cut -f 1 -d "#"`
            if [ $loginname != $HOST ]; then
              $remoteshell $loginname "$scriptdir/rsingle.sh $dir $machine $WORKDIR $DPLDIR $scratch" &
            else
                                       $scriptdir/rsingle.sh $dir $machine $WORKDIR $DPLDIR $scratch &
              #################################################
              cd $DPLDIR
            fi
          fi
        done
      fi
      sleep 1
      # wait until there are free hosts
      while [ `ls -l lockhost.* 2>/dev/null | wc -l` -ge $MAXJOBS ]; do
        sleep 1
      done
    fi
   
    # check whether a stop file exists
    if [ -f stop -o -f STOP ]; then
      rm -f stop STOP
      echo "FOUND stop FILE. TERMINATING..."
      exit 0
    fi
   
    i=`expr $i + 1`
  done
  # end of loop

  wait

  sleep 1
}
############################################################





############################################################
##################              ############################
################## MAIN PROGRAM ############################
##################              ############################
############################################################

#1
# Function
StartJob
echo
echo "Start Job"
date '+Time: %Y-%m-%d %T'

#2
############################################################
while [ $# -gt 0 ]; do
  case "$1" in
    -mfile     )     mfile="$2"; shift; shift ;;
    -ifile     )     ifile="$2"; shift; shift ;;
    -scratch   )   scratch="$2"; shift; shift ;;
    -scriptdir ) scriptdir="$2"; shift; shift ;;
    *          ) cat <<END ;
   
END
    exit;;
  esac
done

MAXJOBS=`cat $mfile| wc -l`

echo "There are $MAXJOBS processors!"

[ -z $ifile -o ! -f $ifile ] && echo "input file !!!" && exit 0

if [ "$MAXJOBS" -gt 1 ]
then
  if [ -z $mfile ]
  then
    echo
    echo "Nodes file not be found!"
    echo "Stop!"
    echo
    exit 0
  fi
fi
[ "$MAXJOBS" -gt "1" -a ! -f $mfile ] && echo "nodes file!!!" && exit 0
############################################################

cd $scriptdir
scriptdir=`pwd`
cd $WORKDIR

if [ ! -d $DPLDIR ]; then
  mkdir $DPLDIR
fi

$scriptdir/dipolecoord < $ifile

#3
# Function
CheckPara
echo
echo "CheckPara Done !!!"
date '+Time: %Y-%m-%d %T'
echo

#4
# Function
DoDipole
echo
echo "DoDipole Done !!!"
date '+Time: %Y-%m-%d %T'
echo

cd $DPLDIR
$scriptdir/ddipole $dfile

echo
echo "... all steps done!"
echo

exit 0
