#!/usr/bin/env python
#
# MOMAP toolkit, http://momap.net.cn
# Converted from numfreq.sh (by Yingli Niu)
# By Qikai Li, qkli@tsinghua.edu.cn, Tsinghua University
#
# Copyright (2018) Zhigang Shuai Group, Tsinghua University.

import os, sys
import time
import glob
import socket
from datetime import timedelta

verbose = False

# =============================================================================
# 1 Start Job

# -----------------------------------------------------------------------
# Print title
sys.stdout.write("  __  __    ___    __  __      _      ____  \n")
sys.stdout.write(" |  \/  |  / _ \  |  \/  |    / \    |  _ \ \n")
sys.stdout.write(" | |\/| | | | | | | |\/| |   / _ \   | |_) |\n")
sys.stdout.write(" | |  | | | |_| | | |  | |  / ___ \  |  __/ \n")
sys.stdout.write(" |_|  |_|  \___/  |_|  |_| /_/   \_\ |_|       Version 2.0.0\n")
sys.stdout.write("\n")
sys.stdout.write(" Copyright (c) 2017 Shuaigroup @ Tsinghua University &\n")
sys.stdout.write("         Institute of Chemistry, Chinese Academy of Sciences.\n")
sys.stdout.write("         All Rights Reserved.\n")
sys.stdout.write("\n")
sys.stdout.write("**************************************************************\n")
sys.stdout.write("*                          numfreq.py                        *\n")
sys.stdout.write("**************************************************************\n")
sys.stdout.flush()

start_time = time.time()

if os.path.exists("finished"):
    sys.stdout.write("\n")
    sys.stdout.write("The job has been done!\n")
    sys.stdout.write("\n")
    sys.exit(0)

WORKDIR = os.getcwd()
PESDIR = WORKDIR + '/PES'
ncpu_all = 1
remoteshell = "ssh"
HOST0 = socket.gethostname()
GetCoordProgram = "none"
GetEnergyProgram = "none"

qctype = "gaussian"
qcpath = "none"
mpipath = "none"
ex = 1
machinefile = "nodefile"
ifile = "input"
scratch = "/tmp"
recalc = "no"
task = "numfreq"
coord = "yes"
scan = "yes"
energy = "yes"
checkpara = "yes"
save_chk = "no"
save_fchk = "no"
ncore = 8
test_rs = "no"
chkfile = "none"
search_chk = "no"
ignore_log_err = "yes"
lock = "lock"
stop = "stop"

sys.stdout.write("--------------------------------------------------------------\n")
sys.stdout.flush()

# =============================================================================
# 2 Get Arguments

sys.stdout.write("Parsing Arguments...\n")

def getopts(argv):
    opts = {}                           # Empty dictionary to store key-value pairs.
    while argv:                         # While there are arguments left to parse...
        if argv[0][0] == '-':           # Found a "-name value" pair.
            opts[argv[0]] = argv[1]     # Add key and value to the dictionary.
        argv = argv[1:]                 # Reduce the argument list by copying it starting from index 1.
    return opts

args = getopts(sys.argv)

args_items = ()
if (sys.version_info > (3, 0)):
    args_items = args.items()          # Python 3.x
else:
    args_items = args.iteritems()      # Python 2.x

for key, val in args_items:
    if key == "-qctype":
        qctype = val
    elif key == "-qcpath":
        qcpath = val
    elif key == "-mpipath":
        mpipath = val
    elif key == "-ex":
        ex = int(val)
    elif key == "-nodefile":
        machinefile = val
    elif key == "-input":
        ifile = val
    elif key == "-scratch":
        scratch = val
    elif key == "-recalc":
        recalc = val
    elif key == "-task":
        task = val
    elif key == "-coord":
        coord = val
    elif key == "-scan":
        scan = val
    elif key == "-energy":
        energy = val
    elif key == "-checkpara":
        checkpara = val
    elif key == "-save_chk":
        save_chk = val
    elif key == "-save_fchk":
        save_fchk = val
    elif key == "-ncore":
        ncore = int(val)
    elif key == "-test_rs":
        test_rs = val
    elif key == "-chkfile":
        chkfile = val
    elif key == "-search_chk":
        search_chk = val
    elif key == "-ignore_log_err":
        ignore_log_err = val
    elif key == "-lock":
        lock = val
    elif key == "-stop":
        stop = val

scriptdir,runname = os.path.split(os.path.abspath(sys.argv[0]))
sys.stdout.write("scriptdir = %s\n" % scriptdir)

sys.stdout.write("\n")
sys.stdout.write("Parsing Arguments Done!\n")
sys.stdout.write("--------------------------------------------------------------\n")
sys.stdout.flush()

# =============================================================================
# 3 Check Arguments

sys.stdout.write("Check Arguments...\n")

ii = ""

initcoord = ""
scantype = ""
imode = "0"

with open(ifile, "r") as f:
    lines = f.readlines()
    for line in lines:
        line = line.strip()
        items = line.split("=")
        token = items[0].lower().strip()

        if token == "inicoord":
            initcoord = items[1].strip()
        elif token == "scantype":
            scantype = items[1].strip()
        elif token == "imode":
            imode = items[1].strip()

    if initcoord:
        ii = initcoord
    elif scantype.lower() == "manual":
        ii = scantype
    else:
        ii = imode

PESDIR = WORKDIR + '/PES-' + ii
if not os.path.exists(PESDIR):
    os.mkdir(PESDIR)

with open(machinefile, "r") as f:
    lines = f.readlines()
    ncpu_all = len(lines)

print ("There are %s processors!" % ncpu_all)

if not os.path.exists(ifile):
    print("Input file not found!")
    sys.exit(1)

if ncpu_all:
    if not os.path.exists(machinefile):
        sys.stderr.write("Node file not found\n");
        sys.stderr.write("STOP\n")
        sys.exit(1)

numfreq_single = scriptdir + '/numfreq_single.py'

sys.stdout.write("Running Parameters of numfreq.py:\n")
sys.stdout.write("  qctype         = %s\n" % qctype)
sys.stdout.write("  qcpath         = %s\n" % qcpath)
sys.stdout.write("  mpipath        = %s\n" % mpipath)
sys.stdout.write("  ex             = %d\n" % ex)
sys.stdout.write("  nodefile       = %s\n" % machinefile)
sys.stdout.write("  input          = %s\n" % ifile)
sys.stdout.write("  scratch        = %s\n" % scratch)
sys.stdout.write("  recalc         = %s\n" % recalc)
sys.stdout.write("  task           = %s\n" % task)
sys.stdout.write("  coord          = %s\n" % coord)
sys.stdout.write("  scan           = %s\n" % scan)
sys.stdout.write("  energy         = %s\n" % energy)
sys.stdout.write("  checkpara      = %s\n" % checkpara)
sys.stdout.write("  save_chk       = %s\n" % save_chk)
sys.stdout.write("  sava_fchk      = %s\n" % save_fchk)
sys.stdout.write("  ncore          = %d\n" % ncore)
sys.stdout.write("  test_rs        = %s\n" % test_rs)
sys.stdout.write("  chkfile        = %s\n" % chkfile)
sys.stdout.write("  ignore_log_err = %s\n" % ignore_log_err)
sys.stdout.write("  lock           = %s\n" % lock)
sys.stdout.write("  stop           = %s\n" % stop)

sys.stdout.write("\n")
sys.stdout.write("Check Arguments Done!\n")
sys.stdout.write("--------------------------------------------------------------\n")
sys.stdout.flush()

# =============================================================================
# 4 Generate coord files

if coord.lower() == "yes":
    sys.stdout.write("Generate Coordinate File...\n")

    os.chdir(WORKDIR)

    if qctype.lower() == "gaussian":
        if task.lower() == "none":
            if inicoord:
                GetCoordProgram = scriptdir + "/gaussian_pes_intcoord.exe " + ifile
            elif scantype == "manual":
                GetCoordProgram = scriptdir + "/gaussian_pes_mancoord.exe " + ifile
        elif task == "mode":
            GetCoordProgram = scriptdir + "/gaussian_pes_mode_coord.exe " + ifile
        elif task == "numfreq":
            GetCoordProgram = scriptdir + "/numfreq_coord.exe " + ifile
        elif task == "pes2":
            GetCoordProgram = scriptdir + "/gaussian_pes2_coord.exe " + ifile
        elif task == "int":
            GetCoordProgram = scriptdir + "/gaussian_pes_int_coord.exe " + ifile
        elif task == "afm":
            GetCoordProgram = scriptdir + "/scan_afm_coord.exe " + qctype + " " + ifile
        elif task == "cluster":
            GetCoordProgram = scriptdir + "/gaussian_cluster_coord.exe " + ifile
        else:
            sys.stderr.write("Input error!\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)
    elif qctype == "turbomole":
        if task == "numf":
            GetCoordProgram = scriptdir + "/numfreq_coord.exe " + qctype + " " + ifile
    elif qctype == "molpro":
        GetCoordProgram = scriptdir + "/scan_mode_coord.exe " + qctype + " " + ifile
    elif qctype == "vasp":
        if task == "afm":
            GetCoordProgram = scriptdir + "/scan_afm_coord.exe " + ifile
        else:
            GetCoordProgram = scriptdir + "/vasp_coord.exe " + ifile
    elif qctype == "nwchem":
        GetCoordProgram = scriptdir + "/scan_afm_coord.exe " + ifile
    elif qctype == "pwscf":
        GetCoordProgram = scriptdir + "/pwscf_coord.exe " + ifile
    elif qctype == "wien2k":
        if task == "density":
            GetCoordProgram = scriptdir + "/wien2k_coord.exe " + ifile
        else:
            sys.stderr.write("Error: task %s is not implemented!\n" % task)
            sys.exit(1)
    else:
        sys.stderr.write("Error: qctype %s is not implemented!\n" % qctype)
        sys.exit(1)

    os.chdir(WORKDIR)

    sys.stdout.write("$ %s\n" % GetCoordProgram)
    ret = os.system(GetCoordProgram)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    sys.stdout.write("\n")
    sys.stdout.write("Generate Coordinate File Done!\n")
    sys.stdout.write("--------------------------------------------------------------\n")
    sys.stdout.flush()

# =============================================================================
# 5 Check Parallel stuff

nmach = 0
nmach_uniq = 0
ncpu_all = 0
ncpu_calc = 0
lines_uniq = []
cpu_uniq = []
cpu_calc = []
cpu_all = []

if checkpara.lower() == "yes":
    sys.stdout.write("Check Parallel...\n")

    if os.path.exists(machinefile):
        with open(machinefile, "r") as f:
            lines = f.read().splitlines()
            nmach = len(lines)
            lines_uniq += sorted(set(lines))
            for cpu in lines_uniq:
                nmach_uniq += 1

            for cpu in lines_uniq:
                # ncpu_node=`grep $cpu $machinefile | wc -l`
                ncpu_node = 0
                for line in lines:
                    if cpu in line:
                        ncpu_node += 1

                # node=`echo $cpu | cut -f 1 -d "."`
                node = cpu.strip().split(".")[0]

                if node != HOST0:
                    cmd = remoteshell + " " + node
                    sys.stdout.write("$ %s\n" % cmd)
                    ret = os.system(cmd)
                    if ret:
                        sys.stderr.write("Error in logging into node '%s'\n" % node)
                        sys.stderr.write("Please check your machinefile '%s'\n" % machinefile)
                        sys.exit(1)
                else:
                    sys.stdout.write("Node '%s' is local host\n" % node)

                cpu_uniq.append(node)

                icpu_node = 0
                icpu_calc = 0
                while icpu_node < ncpu_node:
                    ncpu_mod = icpu_node % ncore
                    if ncpu_mod == 0:
                        icpu_calc += 1
                        ncpu_calc += 1
                        i2 = "%02d" % icpu_calc
                        cpu_calc.append(node + "#" + i2)
                    # -----------------------------------
                    icpu_node += 1
                    ncpu_all += 1
                    i2 = "%02d" % icpu_node
                    cpu_all.append(node + "#" + i2)

            if verbose:
                print(cpu_all)

                print("The machinefile is %s" % machinefile)
                print("--------------------------------------")
                for line in lines:
                    print("  %s" % line)
                print("--------------------------------------")

                print("CPUSTEP: %s" % ncore)
                print("HOSTs:")
                print(cpu_calc)
                sys.stdout.write("ncpu_calc = %s\n" % ncpu_calc)
                sys.stdout.write("\n")

            sys.stdout.write("scratch dir: %s\n" % scratch)
            sys.stdout.write("Runing on nodes:\n")
            for nodename in cpu_uniq:
                sys.stdout.write("  %s\n" % nodename)

    else:
        sys.stderr.write("Cannot find machinefile '%s'\n" % machinefile)
        sys.exit(1)

    sys.stdout.write("\n")
    sys.stdout.write("Check Parallel Done!\n")
    sys.stdout.write("--------------------------------------------------------------\n")
    sys.stdout.flush()

# =============================================================================
# 6 Do SCAN

def check_string(fname, sstr):
    n = 0
    if os.path.isfile(fname):
        with open(fname) as f:
            lines = f.readlines()
            for line in lines:
                if line.find(sstr) != -1:
                    n += 1
    return n;

def find_locks(lock, sstr):
    n = 0
    fnms = lock + "*"
    files = glob.glob(fnms)
    for file in files:
        if file.find(sstr):
            n += 1
    return n

"""Wait for running processes to end."""
def wait_running(ncpu_calc):
    nrun = len(glob.glob1(PESDIR, "lock*"))
    while nrun >= ncpu_calc:
        time.sleep(1)
        nrun = len(glob.glob1(PESDIR, "lock*"))


if scan == "yes":
    sys.stdout.write("Do PES Scan...\n")

    os.chdir(PESDIR)

    # clean previous locks
    fnms = lock + "*"
    files = glob.glob(fnms)
    for file in files:
        os.remove(file)

    NDir = 0
    dirs = []
    with open("files.out", "r") as f:
        lines = f.readlines()
        for line in lines:
            if "coord" in line:
                NDir += 1
                dirs.append(line.strip())
    # print(dirs)

    sys.stdout.write("Running Parameters of numfreq_single.py:\n")
    sys.stdout.write("  lockfile     = %s...\n" % lock)
    sys.stdout.write("  WORKDIR      = %s\n" % WORKDIR)
    sys.stdout.write("  PESDIR       = %s\n" % PESDIR)
    sys.stdout.write("  scratch      = %s\n" % scratch)
    sys.stdout.write("  qcpath       = %s\n" % qcpath)
    sys.stdout.write("  ncore        = %d\n" % ncore)
    sys.stdout.write("  save_chk     = %s\n" % save_chk)
    sys.stdout.write("  save_fchk    = %s\n" % save_fchk)
    sys.stdout.write("  orig_chkfile = %s\n" % chkfile)
    sys.stdout.write("  qctype       = %s\n" % qctype)
    sys.stdout.write("  mpipath      = %s\n" % mpipath)
    sys.stdout.write("  search_chk   = %s\n" % search_chk)
    sys.stdout.write("  task         = %s\n" % task)
    sys.stdout.flush()

    i = 1
    init = "yes"
    for d1 in dirs:
        if os.path.exists(stop) and init == "yes":
            sys.stdout.write("FOUND stop FILE. TERMINATING ...\n")
            sys.exit(0)

        init = "no"
        if_calc = "yes"

        logfile = d1 + ".log"
        if os.path.exists(logfile):
            if_calc = "no"

            if qctype.lower() == "gaussian":
                num_Normal = check_string(logfile, ' Normal termination of Gaussian')
                if num_Normal:
                    if_calc = "yes"

                if ignore_log_err == "yes":
                    if_calc = "no"

            elif qctype == "molpro":
                num_Normal = check_string(logfile, "Variable memory released")
                if num_Normal:
                    if_calc = "yes"

                if ignore_log_err == "yes":
                    if_calc = "no"
                
            elif qctype == "turbomole":
                num_Normal = check_string(logfile, "all done")
                if num_Normal:
                    if_calc = "yes"

                if ignore_log_err == "yes":
                    if_calc = "no"
                
            elif qctype == "nwchem":
                num_Normal = check_string(logfile, "GA Statistics")
                if num_Normal:
                    if_calc = "yes"

                if ignore_log_err == "yes":
                    if_calc = "no"
                
            elif qctype == "vasp":
                num_Normal = check_string(logfile, "General timing")
                if num_Normal:
                    if_calc = "yes"

                if ignore_log_err == "yes":
                    if_calc = "no"

        if recalc == "yes":
            if_calc = "yes"

        if if_calc == "no":
            if not os.path.exists(stop):
                sys.stdout.write("\nStep %s was already done (# %d of %s)\n" % (d1, i, NDir))
                sys.stdout.flush()
        else:
            if not os.path.exists(stop):
                sys.stdout.write("\nStep %s started (# %d of %s)\n" % (d1, i, NDir))
                sys.stdout.flush()

                arguments = \
                    d1 + " " + \
                    WORKDIR + " " + \
                    PESDIR + " " + \
                    scratch + " " + \
                    qcpath + " " + \
                    str(ncore) + " " + \
                    save_chk + " " + \
                    save_fchk + " " + \
                    chkfile + " " + \
                    qctype + " " + \
                    mpipath + " " + \
                    search_chk + " " + \
                    task

                if ncpu_all == 1:
                    # serial runs
                    machine = HOSTNAME
                    n1 = find_locks(lock, machine)
                    n2 = find_locks(lock, d1)
                    nn = n1 + n2
                    if nn == 0 and test_rs == "no":
                        lockfile = lock + "_" + machine + "_" + d1
                        cmd = "touch " + lockfile;
                        os.system(cmd)

                        cmd = numfreq_single + " " + lockfile + " " + arguments + " &"
                        if verbose:
                            sys.stdout.write("$ %s\n" % cmd)
                            sys.stdout.flush()
                        ret = os.system(cmd)
                        if ret:
                            sys.stderr.write("Terminated with ERROR\n")
                            sys.stderr.write("STOP\n")
                            sys.exit(1)
                else:
                    # look for a free host
                    for machine in cpu_calc:
                        n1 = find_locks(lock, machine)
                        n2 = find_locks(lock, d1)
                        nn = n1 + n2

                        if nn == 0 and test_rs == "no":
                            node = machine.split('#')[0]
                            lockfile = lock + "_" + machine + "_" + d1
                            cmd = "touch " + lockfile;
                            os.system(cmd)

                            if node == HOST0:
                                cmd = numfreq_single + " " + lockfile + " " + arguments + " &"
                                if verbose:
                                    sys.stdout.write("$ %s\n" % cmd)
                                    sys.stdout.flush()
                                ret = os.system(cmd)
                                if ret:
                                    sys.stderr.write("Terminated with ERROR\n")
                                    sys.stderr.write("STOP\n")
                                    sys.exit(1)
                            else:
                                cmd0 = numfreq_single + " " + lockfile + " " + arguments
                                cmd = remoteshell + " " + node + " " + "'" + cmd0 + "' &"
                                if verbose:
                                    sys.stdout.write("$ %s\n" % cmd)
                                    sys.stdout.flush()
                                ret = os.system(cmd)
                                if ret:
                                    sys.stderr.write("Terminated with ERROR\n")
                                    sys.stderr.write("STOP\n")
                                    sys.exit(1)

            # wait until there are free hosts
            # print(ncpu_calc)
            wait_running(ncpu_calc)
            
        # check whether a stop file exists
        if os.path.exists(stop):
            ncpu_calc -= 1
            if ncpu_calc == 0:
                sys.stdout.write("FOUND stop FILE. TERMINATING...\n")
                cmd = "rm -f %s" % stop
                os.system(cmd)
                sys.exit(1)

        i += 1

    sys.stdout.write("\n")
    sys.stdout.write("Do PES Scan Done!\n")
    sys.stdout.write("--------------------------------------------------------------\n")
    sys.stdout.flush()

# =============================================================================
# 7 Get energy or frequency

if energy == "yes":
    sys.stdout.write("Get energy or frequency...\n")

    os.chdir(PESDIR)

    if qctype == "gaussian":
        if energy != "no":
            if task == "none":
                GetEnergyProgram = scriptdir + "/gaussian_pes_energy.exe"
            elif task == "mode":
                GetEnergyProgram = scriptdir + "/gaussian_pes_mode_energy.exe"
            elif task == "numfreq":
                GetEnergyProgram = scriptdir + "/numfreq_gather.exe ../" + ifile
            elif task == "pes2":
                GetEnergyProgram = scriptdir + "/gaussian_pes2_energy.exe"
            elif task == "int":
                GetEnergyProgram = scriptdir + "/gaussian_pes_int_energy.exe"
            elif task == "cluster":
                GetEnergyProgram = scriptdir + "/gaussian_pes_int_energy.exe"
            elif task == "afm":
                GetEnergyProgram = scriptdir + "/numfreq_gather.exe"
            else:
                sys.stderr.write("Input error!\n")
                sys.stderr.write("STOP\n")
                sys.exit(1)
    elif qctype == "turbomole":
        if task == "numf":
            GetEnergyProgram = scriptdir + "/scan_numf_energy.exe"
    elif qctype == "molpro":
        GetEnergyProgram = scriptdir + "/scan_mode_energy.exe"
    elif qctype == "vasp":
        if task == "afm":
            GetEnergyProgram = scriptdir + "/scan_afm_energy.exe"
        else:
            GetEnergyProgram = scriptdir + "/vasp_energy.exe"
    elif qctype == "nwchem":
        GetEnergyProgram = scriptdir + "/scan_afm_energy.exe"
    elif qctype == "pwscf":
        GetEnergyProgram = scriptdir + "/pwscf_energy.exe"
    elif qctype == "wien2k":
        GetEnergyProgram = scriptdir + "/wien2k_energy.exe"

    os.chdir(PESDIR)

    sys.stdout.write("Generate Scan Information...\n")
    sys.stdout.write("$ %s\n" % GetEnergyProgram)
    ret = os.system(GetEnergyProgram)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("Current directory: %s\n" % os.getcwd())
        sys.stderr.write("STOP\n")
        sys.exit(1)

    sys.stdout.write("\n")
    sys.stdout.write("Get energy or frequency Done!\n")
    sys.stdout.write("--------------------------------------------------------------\n")
    sys.stdout.flush()

# =============================================================================
# Finshed

# Print message
sys.stdout.write("****************************************\n")
sys.stdout.write("*        ALL SUCCESSFULLY DONE.        *\n")
sys.stdout.write("****************************************\n")
sys.stdout.flush()

# Print elapsed time
seconds = time.time() - start_time

seconds_in_day = 86400
seconds_in_hour = 3600
seconds_in_minute = 60

days = seconds // seconds_in_day
seconds = seconds - (days * seconds_in_day)

hours = seconds // seconds_in_hour
seconds = seconds - (hours * seconds_in_hour)

minutes = seconds // seconds_in_minute
seconds = seconds - (minutes * seconds_in_minute)

print("DURATION: {0:.0f} days {1:.0f} hours {2:.0f} minutes {3:.0f} seconds.".format(
        days, hours, minutes, seconds))

