#!/usr/bin/env python
#
# MOMAP toolkit, http://momap.net.cn
# Converted from numfreq_single.sh (by Yingli Niu)
# By Qikai Li, qkli@tsinghua.edu.cn, Tsinghua University
#
# Copyright (2018) Zhigang Shuai Group, Tsinghua University.
#
# In normal situation, this script is to be called by numfreq.py
#

import os, sys
import subprocess
import time
import glob
import fileinput
import shutil
from tempfile import mkstemp
import platform

py_version = platform.python_version()
py_ver = 2

sys.stdout.write("**************************************************************\n")
sys.stdout.write("*                       numfreq_single.py                    *\n")
sys.stdout.write("**************************************************************\n")

lockfile = sys.argv[1]
fstub = sys.argv[2]         # formal variable dir
WORKDIR = sys.argv[3]
PESDIR = sys.argv[4]
scratch = sys.argv[5]
qcpath = sys.argv[6]
ncore = int(sys.argv[7])
save_chk = sys.argv[8]
save_fchk = sys.argv[9]
orig_chkfile = sys.argv[10]
qctype = sys.argv[11]
mpipath = sys.argv[12]
search_chk = sys.argv[13]
task = sys.argv[14]

verbose = False

if verbose:
    sys.stdout.write("Running numfreq_single.py...\n")
    sys.stdout.write("Running Parameters:\n")
    sys.stdout.write("  lockfile     = %s\n" % lockfile)
    sys.stdout.write("  fstub        = %s\n" % fstub)
    sys.stdout.write("  WORKDIR      = %s\n" % WORKDIR)
    sys.stdout.write("  PESDIR       = %s\n" % PESDIR)
    sys.stdout.write("  scratch      = %s\n" % scratch)
    sys.stdout.write("  qcpath       = %s\n" % qcpath)
    sys.stdout.write("  ncore        = %d\n" % ncore)
    sys.stdout.write("  save_chk     = %s\n" % save_chk)
    sys.stdout.write("  save_fchk    = %s\n" % save_fchk)
    sys.stdout.write("  orig_chkfile = %s\n" % orig_chkfile)
    sys.stdout.write("  qctype       = %s\n" % qctype)
    sys.stdout.write("  mpipath      = %s\n" % mpipath)
    sys.stdout.write("  search_chk   = %s\n" % search_chk)
    sys.stdout.write("  task         = %s\n" % task)

sys.stdout.write("Working file stub: %s\n" % fstub)

scriptdir,runname = os.path.split(os.path.abspath(sys.argv[0]))

qcdir,qcexe = os.path.split(qcpath)
os.environ["qcdir"] = qcdir


# Created scratch directory if do not exist

scratch = scratch + "/" + fstub + ".scr"
if not os.path.exists(scratch):
    os.makedirs(scratch)

sys.stdout.write("Scratch directory: %s\n" % scratch)
sys.stdout.flush()


# check the number of lines containing a substring

def check_string(fname, sstr):
    n = 0
    if os.path.isfile(fname):
        with open(fname) as f:
            lines = f.readlines()
            for line in lines:
                if line.find(sstr) != -1:
                    n += 1
    return n;


# =============================================================================
# Gaussian

if qctype == "gaussian":
    os.environ["GAUSS_EXEDIR"] = qcdir
    os.environ["PATH"] += os.pathsep + qcdir

    readchk = 0
    gausopt = 0
    ifforce = 0
    iffreq = 0
    ifnosy = 0

    filecom = PESDIR + "/" + fstub + ".com"

    if verbose:
        sys.stdout.write("Modify Gaussian .com file...\n")

    with open(filecom, "r") as f:
        lines = f.read().splitlines()
        for line in lines:
            line_lc = line.lower()
            if "guess=read" in line_lc:
                readchk += 1
            if "opt" in line_lc:
                gausopt += 1
            if "force" in line_lc:
                ifforce += 1
            if "freq" in line_lc:
                iffreq += 1
            if "nosymm" in line_lc:
                ifnosy += 1

    chkfile = fstub + ".chk"
    checkfile = fstub + ".chk"

    # append the necessary string
    stradd = ""
    if task == "numfreq":
        if ifforce == 0:
            stradd += " force"
        if ifnosy == 0:
            stradd += " nosymm"

    # modify the the # line in gaussian .com file
    if stradd:
        fh, abs_path = mkstemp()
        with os.fdopen(fh, 'w') as new_file:
            with open(filecom) as old_file:
                for line in old_file:
                    if line.lstrip().startswith("#"):
                        new_file.write(line.rstrip() + stradd + "\n")
                    else:
                        new_file.write(line)
        os.remove(filecom)
        shutil.move(abs_path, filecom)

    if verbose:
        sys.stdout.write("Run search_chk...\n")

    if search_chk.lower() == "yes":
        pathstr = PESDIR + "/*.chk"
        files = glob.glob(pathstr)
        if len(files):
            flst = fstub + ".lst"
            with open(flst, 'w+') as f:
                for file in files:
                    f.write(file)

            # this returned fmin needs to checked!!!
            cmd = scriptdir + "/search_nearest_file.exe"

            fmin = "no"

            if py_ver == 2:
                fmin = subprocess.Popen([cmd, fstub, flst], stdout=subprocess.PIPE).communicate()[0]
            else:
                fmin = subprocess.check_output([cmd, fstub, flst])  # version 2.7 or above

            fmin = fmin.strip()

            if verbose:
                sys.stdout.write("INFO: fmin = %s\n" % fmin)

            if fmin != "no":
                # get rid of the extension .chk
                fmin = os.path.splitext(fmin)[0]

                filechk = PESDIR + "/" + fmin + ".chk"
                if os.path.isfile(filechk):
                    # debug
                    fileinf = PESDIR + "/" + fstub + ".inf"
                    with open(fileinf, 'r+') as f:
                        contents = f.read()
                        addstr = 'copy chk: ' + fmin + ".chk -> " + fstub + '.chk\n'
                        f.seek(0, 0)
                        f.write(contents + addstr)

                    dst = scratch + "/" + fstub + ".chk"
                    shutil.copyfile(filechk, dst)

                    # modify line starting with '#' in .com file
                    if readchk == 0:
                        if gausopt == 0:
                            fh, abs_path = mkstemp()
                            with os.fdopen(fh, 'w') as new_file:
                                with open(filecom) as old_file:
                                    for line in old_file:
                                        if line.lstrip().startswith("#"):
                                            new_file.write(line.rstrip() + " guess=read\n")
                                        else:
                                            new_file.write(line)
                            os.remove(filecom)
                            shutil.move(abs_path, filecom)

            os.remove(flst)

    #-------------------------------------------------------------------------------
    nline = check_string(filecom, "%chk")
    if nline == 0:
        with open(filecom, 'r+') as f:
            contents = f.read()
            substr = '%chk=' + checkfile + '\n'
            f.seek(0, 0)
            f.write(substr + contents)
    else:
        fh, abs_path = mkstemp()
        with os.fdopen(fh, 'w') as new_file:
            with open(filecom) as old_file:
                for line in old_file:
                    if line.strip().startswith("%chk"):
                        chkstr = '%chk=' + checkfile + '\n'
                        new_file.write(chkstr)
                    else:
                        new_file.write(line)
        os.remove(filecom)
        shutil.move(abs_path, filecom)

    if verbose:
        sys.stdout.write("Modify Gaussian .com file...\n")

    if search_chk.lower() == "yes":
        pathstr = PESDIR + "/*.chk"
        files = glob.glob(pathstr)
        
        # modify line starting with '#' in .com file
        if readchk and len(files):
            fh, abs_path = mkstemp()
            with os.fdopen(fh, 'w') as new_file:
                with open(filecom) as old_file:
                    for line in old_file:
                        if line.strip().startswith("#"):
                            new_file.write(line.replace('guess=read', ""))
                        else:
                            new_file.write(line)
            os.remove(filecom)
            shutil.move(abs_path, filecom)

    if ncore:
        nline = check_string(filecom, "%nprocs")
        if nline == 0:
            with open(filecom, 'r+') as f:
                contents = f.read()
                prestr = '%nprocs=' + str(ncore) + '\n'
                f.seek(0, 0)
                f.write(prestr + contents)
        else:
            fh, abs_path = mkstemp()
            with os.fdopen(fh, 'w') as new_file:
                with open(filecom) as old_file:
                    for line in old_file:
                        if line.strip().startswith("%nprocs"):
                            substr = '%nprocs=' + str(ncore) + '\n'
                            new_file.write(substr)
                        else:
                            new_file.write(line)
            os.remove(filecom)
            shutil.move(abs_path, filecom)


    sys.stdout.write("Run Gaussian calculation...\n")
    sys.stdout.flush()

    os.chdir(scratch)

    src = PESDIR + "/" + fstub + ".com"
    dst = scratch + "/" + fstub + ".com"
    shutil.copyfile(src, dst)

    # Do Gaussian calculation
    cmd = qcpath + " " + fstub + ".com" + " > " + scratch + "/" + fstub + ".log"
    if verbose:
        sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    logfile = scratch + "/" + fstub + ".log"
    calc_success = check_string(logfile, ' Normal termination of Gaussian')
    if calc_success:
        yes = PESDIR + "/" + fstub + ".yes"
        cmd = "touch " + yes
        if verbose:
            sys.stdout.write("$ %s\n" % cmd)
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)
    else:
        print("Error termination in %s" % logfile)
        err = PESDIR + "/" + fstub + ".err"
        cmd = "touch " + err
        if verbose:
            sys.stdout.write("$ %s\n" % cmd)
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)

    if save_fchk.lower() == "yes" and calc_success:
        cmd = qcdir + "/formchk " + fstub + ".chk >& /dev/null"
        if verbose:
            sys.stdout.write("$ %s\n" % cmd)
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)

        if verbose:
            sys.stdout.write("Copying .fchk file from scratch to PESDIR...\n")

        src = scratch + "/" + fstub + ".fchk"
        if os.path.isfile(src):
            dst = PESDIR + "/" + fstub + ".fchk"
            shutil.move(src, dst)

    if save_chk.lower() == "yes" and calc_success:
        if verbose:
            sys.stdout.write("Copying .chk file from scratch to PESDIR...\n")

        src = scratch + "/" + fstub + ".chk"
        dst = PESDIR + "/" + fstub + ".chk"
        shutil.move(src, dst)

    if verbose:
        sys.stdout.write("Copying .log files from scratch to PESDIR...\n")

    #src = scratch + "/" + fstub + ".com"
    #dst = PESDIR + "/" + fstub + ".com"
    #shutil.move(src, dst)

    src = scratch + "/" + fstub + ".log"
    dst = PESDIR + "/" + fstub + ".log"
    shutil.move(src, dst)

# =============================================================================
# Turbomole

elif qctype == "turbomole":
    TURBODIR = "/export/home/turbomole/TURBOMOLE"
    os.environ["TURBODIR"] = TURBODIR
    os.environ["PATH"] += os.pathsep + TURBODIR + "/bin/em64t-unknown-linux-gnu"
    os.environ["PATH"] += os.pathsep + TURBODIR + "/scripts"

    os.environ["PARNODES"] = ncore
# 	os.environ["PARA_ARCH"] = "SMP"
# 	os.environ["PARA_ARCH"] = "MPI"

    os.chdir(WORKDIR)
    
    cmd = "cpc " + scratch
    print(cmd)
    os.system(cmd)

    cmd = "cp " + WORKDIR + "/sing_a " + scratch + "/sing_a"
    print(cmd)
    os.system(cmd)

    os.chdir(scratch)

    cmd = "cp " + PESDIR + "/" + fstub + ".com " + scratch + "/coord"
    print(cmd)
    os.system(cmd)

    cmd = qcdir + "/dscf > " + fstub + ".log"
    sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    cmd = qcdir + "/egrad >> " + fstub + ".log"
    sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    cmd = "mv " + scratch + "/" + fstub + ".log " + PESDIR + "/" + fstub + ".log"
    print(cmd)
    os.system(cmd)

# =============================================================================
# MOLPRO

elif qctype.lower() == "molpro":
    os.environ["MOLPRO_SCRDIR"] = scratch
    os.environ["TMPDIR"] = scratch

    os.chdir(scratch)

    src = PESDIR + "/" + fstub + ".com"
    dst = scratch + "/" + fstub + ".com"
    cmd = "cp " + src + " " + dst
    os.system(cmd)

    cmd = qcpath + " " + fstub + ".com"
    sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    src = scratch + "/" + fstub + ".com"
    dst = PESDIR + "/" + fstub + ".com"
    shutil.move(src, dst)

    src = scratch + "/" + fstub + ".out"
    dst = PESDIR + "/" + fstub + ".log"
    shutil.move(src, dst)

# =============================================================================
# VASP

elif qctype.lower() == "vasp":
    if search_chk.lower() == "yes":
        pathstr = PESDIR + "/" + "*.chk"
        files = glob.glob(pathstr)
        if len(files):
            flst = fstub + ".lst"
            with open(flst) as f:
                for file in files:
                    f.write(file)

            cmd = scriptdir + "/search_nearest_file.exe " + fstub + " " + flst
            fmin = os.system(cmd)

            filechk = PESDIR + "/" + fmin + ".chk"
            if os.path.isfile(filechk):
                # debug
                fileinf = PESDIR + "/" + fstub + ".inf"
                with open(fileinf, 'r+') as f:
                    contents = f.read()
                    addstr = 'copy chk: ' + fmin + ".chk -> " + fstub + '.chk\n'
                    f.seek(0, 0)
                    f.write(contents + addstr)

                dst = scratch + "/WAVECAR"
                shutil.copyfile(filechk, dst)

            os.remove(flst)

    cmd = "cp " + WORKDIR + "/INCAR " + scratch + "/INCAR"
    os.system(cmd)

    cmd = "cp " + WORKDIR + "/POTCAR " + scratch + "/POTCAR"
    os.system(cmd)

    cmd = "cp " + PESDIR + "/" + fstub + ".com " + scratch + "/POSCAR"
    os.system(cmd)

    # cmd = "cp " + PESDIR + "/" + fstub + ".chk " + scratch + "/WAVECAR"
    # os.system(cmd)

    os.chdir(scratch)

    if ncore == 1:
        os.system(qcpath)
    elif ncore > 1:
        if not os.path.exist("~/.mpd.conf"):
            with open("~/.mpd.conf") as f:
                f.write("secretword=dfadfs")
        cmd = "/bin/chmod 600 ~/.mpd.conf"
        os.system(cmd)

        os.remove("hosts")
        icore = 0
        with open("hosts") as f:
            while icore < ncore:
                f.write("localhost\n")
                icore += 1

        N_MPD = 1

        cmd = mpipath + "/bin64/mpdboot -r ssh -n " + N_MPD + " -f hosts"
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)

        cmd = mpipath + "/bin64/mpiexec -machinefile hosts -n " + str(ncore) + " " + qcpath + " > OUTLOG"
        sys.stdout.write("$ %s\n" % cmd)
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)

        src = scratch + "/OUTCAR"
        dst = PESDIR + "/" + fstub + ".log"
        shutil.move(src, dst)

        if save_chk.lower() == "yes":
            src = scratch + "/OUTLOG"
            dst = PESDIR + "/" + fstub + ".out"
            shutil.move(src, dst)

            src = scratch + "/WAVECAR"
            dst = PESDIR + "/" + fstub + ".chk"
            shutil.move(src, dst)

        cmd = mpipath + "/bin64/mpdallexit"
        sys.stdout.write("$ %s\n" % cmd)
        ret = os.system(cmd)
        if ret:
            sys.stderr.write("Terminated with ERROR\n")
            sys.stderr.write("STOP\n")
            sys.exit(1)

# =============================================================================
# PWSCF

elif qctype.lower() == "pwscf":
    os.chdir(scratch)

    os.remove("hosts")
    icore = 0
    with open("hosts") as f:
        while icore < ncore:
            f.write("localhost\n")
            icore += 1

    cmd = "cp " + PESDIR + "/" + fstub + ".com " + scratch
    sys.stdout.write("$ %s\n" % cmd)
    os.system(cmd)

    cmd = "cp -r " + PESDIR + "/pseudo " + scratch
    sys.stdout.write("$ %s\n" % cmd)
    os.system(cmd)

    cmd = mpipath + "/bin/mpirun -machinefile " + scratch + "/hosts -np " + str(ncore) \
        + " " + qcpath + "-npool 1 < " + fstub + ".com > " + fstub + ".log"
    sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    src = scratch + "/" + fstub + ".log"
    dst = PESDIR + "/" + fstub + ".log"
    shutil.filecopy(src, dst)

# =============================================================================
# NWCHEM

elif qctype.lower() == "nwchem":
    os.chdir(scratch)

    src = PESDIR + "/" + fstub + ".com"
    dst = scratch + "/" + fstub + ".com"
    shutil.filecopy(src, dst)

    if not os.path.exist("~/.mpd.conf"):
        with open("~/.mpd.conf") as f:
            f.write("secretword=dfadfs")
    cmd = "/bin/chmod 600 ~/.mpd.conf"
    os.system(cmd)

    # Intel MPI Home
    MPI_HOME = mpipath
    os.environ["MPICH_PROCESS_GROUP"] = no

    os.remove("hosts")
    icore = 0
    with open("hosts") as f:
        while icore < ncore:
            f.write("localhost\n")
            icore += 1

    # N_MPD=1
    # $mpipath/mpdboot -r ssh -n $N_MPD -f hosts

    if search_chk.lower() == "yes":
        # debug
        pathstr = PESDIR + "/" + "*.chk"
        files = glob.glob(pathstr)
        flst = fstub + ".lst"
        if len(files):
            with open(flst) as f:
                for file in files:
                    f.write(file)

        cmd = scriptdir + "/search_nearest_file.exe " + fstub + " " + flst
        print(cmd)
        fmin = os.system(cmd)

        filechk = PESDIR + "/" + fmin + ".chk"
        if os.path.isfile(filechk):
            # debug
            fileinf = PESDIR + "/" + fstub + ".inf"
            with open(fileinf, 'r+') as f:
                contents = f.read()
                addstr = "copy chk: " + fmin + ".chk -> " + fstub + ".chk\n"
                f.seek(0, 0)
                f.write(contents + addstr)

            dst = scratch + "/WAVECAR"
            shutil.copyfile(filechk, dst)

        os.remove(flst)

    filecom = fstub + ".com"
    nline = check_string(filecom, "vectors output")
    if nline:
        fh, abs_path = mkstemp()
        with os.fdopen(fh, 'w') as new_file:
            with open(filecom) as old_file:
                for line in old_file:
                    if line.strip().find("vectors output"):
                        substr = "vectors output \"" + fstub + ".chk\""
                        new_file.write(substr)
                    else:
                        new_file.write(line)
        os.remove(filecom)
        shutil.move(abs_path, filecom)

    nline = check_string(filecom, "vectors input")
    filechk = PESDIR + "/" + fstub + ".chk"
    exist_chk = os.path.isfile(filechk)
    if exist_check and nline == 1:
        fh, abs_path = mkstemp()
        with os.fdopen(fh, 'w') as new_file:
            with open(filecom) as old_file:
                for line in old_file:
                    if line.strip().find("vectors input"):
                        substr = "vectors input \"" + fstub + ".chk\""
                        new_file.write(substr)
                    else:
                        new_file.write(line)
        os.remove(filecom)
        shutil.move(abs_path, filecom)

    elif not exist_chk and nline == 1:
        fh, abs_path = mkstemp()
        with os.fdopen(fh, 'w') as new_file:
            with open(filecom) as old_file:
                for line in old_file:
                    if not line.strip().find("vectors input"):
                        new_file.write(line)
        os.remove(filecom)
        shutil.move(abs_path, filecom)

    cmd = mpipath + "/mpiexec -machinefile hosts -n " + str(ncore) + " " + qcpath + " " \
        + fstub + ".com > " + fstub + ".log 2> " + fstub + ".err"
    sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    nline = check_string(filecom, "vectors input")
    mline = check_string(filecom, "vectors output")
    if nline == 0 and mline == 1:
        fh, abs_path = mkstemp()
        with os.fdopen(fh, 'w') as new_file:
            with open(filecom) as old_file:
                for line in old_file:
                    if line.strip().find("vectors output"):
                        addstr = "vectors input \"" + fstub + ".chk\""
                        new_file.write(addstr)
                        new_file.write(line)
                    else:
                        new_file.write(line)
        os.remove(file_path)
        shutil.move(abs_path, filecom)

    cmd = "cp " + scratch + "/" + fstub + ".com " + PESDIR + "/" + fstub + ".com"
    os.system(cmd)

    cmd = "cp " + scratch + "/" + fstub + ".log " + PESDIR + "/" + fstub + ".log"
    os.system(cmd)

    cmd = "cp " + scratch + "/" + fstub + ".err " + PESDIR + "/" + fstub + ".err"
    os.system(cmd)

    if ret == 0:
        cmd = "cp " + scratch + "/" + fstub + ".inf " + PESDIR + "/" + fstub + ".inf"
        os.system(cmd)

        cmd = "cp " + scratch + "/" + fstub + ".chk " + PESDIR + "/" + fstub + ".chk"
        os.system(cmd)

# =============================================================================
# WIEN2K

elif qctype.lower() == "wien2k":
    os.chdir(PESDIR)

    files = ""
    fout5 = ""
    frho1 = ""

    filedef = fstub + ".def"
    with open(filedef) as f:
        lines = f.read().splitlines()
        for line in lines:
            items = line.strip().split(',')
            if "old" in line:
                files += items[1].strip("'") + " "
            if "coord" in line:
                if "rho" in line:
                    frho1 = items[1].strip("'")
                if "log" in line:
                    foutt = items[1].strip("'")

    cmd = "cp " + fstub + ".def " + files + " " + scratch
    sys.stdout.write("$ %s\n" % cmd)
    os.system(cmd)

    os.chdir(scratch)

    cmd = "lapw5 " + fstub + ".def"
    sys.stdout.write("$ %s\n" % cmd)
    ret = os.system(cmd)
    if ret:
        sys.stderr.write("Terminated with ERROR\n")
        sys.stderr.write("STOP\n")
        sys.exit(1)

    cmd = "cp " + frho1 + " " + fout5 + " " + PESDIR
    sys.stdout.write("$ %s\n" % cmd)
    os.system(cmd)

# =============================================================================
# Finally, clean the scratch and lock

# To get rid of message: shell-init: error retrieving current directory: getcwd: ...
os.chdir(WORKDIR)

cmd = "rm -rf " + scratch
if verbose:
    sys.stdout.write("$ %s\n" % cmd)
os.system(cmd)

cmd = "rm -f " + PESDIR + "/" + lockfile
if verbose:
    sys.stdout.write("$ %s\n" % cmd)
os.system(cmd)

sys.stdout.write("Normal End of numfreq_single.py.\n")
sys.stdout.flush()

