#!/usr/bin/env python
import os,sys
from datetime import datetime

#------------------------------------+
#  MOMAP File Conversion Utility     |
#  Website: http://www.momap.net.cn  |
#------------------------------------+

'''
PDB Record Format

COLUMNS        DATA  TYPE    FIELD        DEFINITION
-------------------------------------------------------------------------------------
   1 -  6        Record name   "ATOM  "
   7 - 11        Integer       serial       Atom  serial number.
  13 - 16        Atom          name         Atom name.
  17             Character     altLoc       Alternate location indicator.
  18 - 20        Residue name  resName      Residue name.
  22             Character     chainID      Chain identifier.
  23 - 26        Integer       resSeq       Residue sequence number.
  27             AChar         iCode        Code for insertion of residues.
  31 - 38        Real(8.3)     x            Orthogonal coordinates for X in Angstroms.
  39 - 46        Real(8.3)     y            Orthogonal coordinates for Y in Angstroms.
  47 - 54        Real(8.3)     z            Orthogonal coordinates for Z in Angstroms.
  55 - 60        Real(6.2)     occupancy    Occupancy.
  61 - 66        Real(6.2)     tempFactor   Temperature  factor.
  77 - 78        LString(2)    element      Element symbol, right-justified.
  79 - 80        LString(2)    charge       Charge  on the atom.
'''

if len(sys.argv) == 1:
    print("Usage: mol2pdb.py input.mol")
    sys.exit(1)

Input = sys.argv[1]
Output = os.path.splitext(Input)[0] + '.pdb'

print("Converting %s..." % Input)

out_lines = ""
out_nline = 0

natoms = 0
atomnms = []
coord_x = []
coord_y = []
coord_z = []

if os.path.isfile(Input):
    with open(Input) as f:
        lines = f.readlines()
        ntot = len(lines)

        nbeg = 3
        line = lines[nbeg];
        natoms = int(line.split()[0])
        print("natoms = %d" % natoms)

        nbeg = 4
        for i in range(0, natoms):
            line = lines[nbeg + i]
            items = line.split()
            coord_x.append(float(items[0]))
            coord_y.append(float(items[1]))
            coord_z.append(float(items[2]))
            atomnms.append(items[3])


with open(Output, 'w') as f:
    today = datetime.now()
    f.write("REMARK   MOMAP PDB file\n")
    f.write("REMARK   Created:  %s\n" % (today.strftime("%c")))
    resnm = "MOL"
    resnr = 1
    for i in range(0, natoms):
        ai = i + 1
        nm = atomnms[i]
        x = coord_x[i]
        y = coord_y[i]
        z = coord_z[i]
        f.write("%-6s%5d %2s%-2d %3.3s %1s%4d    %8.3f%8.3f%8.3f%6.2f%6.2f%10s%2s%2s\n" % 
                ("ATOM", ai, nm, ai, resnm, "", resnr, x, y, z, 1.0, 0.0, "", nm, ""))
    f.write("TER\n")

print("to %s done." % Output)

