#!/usr/bin/env python

import os
import sys
import math

# output plot dimension

width = 1200
height = 800

# root and its neighbors info

root_nmols = 0
root_nneis = []

root_types = []
root_x = []
root_y = []
root_z = []

nei_types = []
nei_dx = []
nei_dy = []
nei_dz = []
nei_dr = []

with open("neighbor.dat", "r") as f:
    lines = f.readlines()
    idx = 2;
    root_nmols = int(lines[idx])
    idx += 1

    # print(root_nmols)

    for i in range(0, root_nmols):
        line = lines[idx]
        idx += 1
        items = line.split()

        root_types.append(int(items[0]))
        root_x.append(float(items[1]))
        root_y.append(float(items[2]))
        root_z.append(float(items[3]))

        line = lines[idx]
        idx += 1

        items = line.split()
        nnei = int(items[0])
        root_nneis.append(nnei)

        # print(nnei)

        for j in range(0, nnei):
            line = lines[idx]
            idx += 1
            items = line.split()
            nei_types.append(int(items[0]))
            nei_dx.append(float(items[1]))
            nei_dy.append(float(items[2]))
            nei_dz.append(float(items[3]))
            nei_dr.append(float(items[4]))

    #print(root_nneis)
    #print(root_types)
    #print(root_x)
    #print(root_y)
    #print(root_z)

    #print(nei_types)
    #print(nei_dx)
    #print(nei_dy)
    #print(nei_dz)
    #print(nei_dr)


def find_scale_factor():
    xmin = ymin = zmin = 1e20
    xmax = ymax = zmax = -1e20
    xlim = ylim = zlim = 0.0
    s1 = s2 = 1.0

    k = 0
    for i in range(0, root_nmols):
        x0 = root_x[i]
        y0 = root_y[i]
        z0 = root_z[i]

        for j in range(0, root_nneis[i]):
            x = nei_dx[k] + x0
            y = nei_dy[k] + y0
            z = nei_dz[k] + z0
            k += 1

            if (x > xmax):
                xmax = x
            if (y > ymax):
                ymax = y
            if (z > zmax):
                zmax = z

            if (x < xmin):
                xmin = x
            if (y < ymin):
                ymin = y
            if (z < zmin):
                zmin = z

    xlim = xmax - xmin
    ylim = ymax - ymin
    zlim = zmax - zmin

    if xlim > ylim:
        ylim = xlim
    else:
        xlim = ylim


    s1 = height / xlim * 0.95
    s2 = width  / zlim * 0.95
    return [s1,s2]

# cell vectors a, b, and c for axle drawing

latt_a = [1.0, 0.0, 0.0]
latt_b = [0.0, 1.0, 0.0]
latt_c = [0.0, 0.0, 1.0]

vec_a = [1.0, 0.0, 0.0]
vec_b = [0.0, 1.0, 0.0]
vec_c = [0.0, 0.0, 1.0]

with open("cellvec.dat", "r") as f:
    lines = f.readlines()

    line = lines[0]
    items = line.split()
    latt_a[0] = float(items[2])
    latt_a[1] = float(items[3])
    latt_a[2] = float(items[4])

    line = lines[1]
    items = line.split()
    latt_b[0] = float(items[2])
    latt_b[1] = float(items[3])
    latt_b[2] = float(items[4])

    line = lines[2]
    items = line.split()
    latt_c[0] = float(items[2])
    latt_c[1] = float(items[3])
    latt_c[2] = float(items[4])

    sr = math.sqrt(latt_a[0]*latt_a[0] + latt_a[1]*latt_a[1] + latt_a[2]*latt_a[2])
    vec_a[0] = latt_a[0] / sr
    vec_a[1] = latt_a[1] / sr
    vec_a[2] = latt_a[2] / sr

    sr = math.sqrt(latt_b[0]*latt_b[0] + latt_b[1]*latt_b[1] + latt_b[2]*latt_b[2])
    vec_b[0] = latt_b[0] / sr
    vec_b[1] = latt_b[1] / sr
    vec_b[2] = latt_b[2] / sr

    sr = math.sqrt(latt_c[0]*latt_c[0] + latt_c[1]*latt_c[1] + latt_c[2]*latt_c[2])
    vec_c[0] = latt_c[0] / sr
    vec_c[1] = latt_c[1] / sr
    vec_c[2] = latt_c[2] / sr


# common header part of .pov

def write_header(filename):
    with open(filename, "w") as o:
        o.write("#version 3.7;\n")
        o.write("#include \"colors.inc\"\n")
        o.write("\n")
        o.write("// NOTE: if you plan to render at a different resolution,\n")
        o.write("// be sure to update the following two lines to maintain\n")
        o.write("// the correct aspect ratio.\n")
        o.write("#declare Width = %d;\n" % width)
        o.write("#declare Height = %d;\n" % height)
        o.write("#declare minScreenDimension = %d;\n" % height)
        o.write("#declare showAtoms = true;\n")
        o.write("#declare showBonds = true;\n")
        o.write("#declare noShadows = true;\n")
        o.write("#declare font_scale = 40;\n")
        o.write("#declare font_name = \"cyrvetic.ttf\";\n")
        o.write("\n")
        o.write("global_settings { assumed_gamma 1.0 }\n")
        o.write("\n")
        o.write("camera{\n")
        o.write("   orthographic\n")
        o.write("   location < %g, %g, 0>\n" % (height * 0.74, height * 0.5))
        o.write("\n")
        o.write("   // Negative right for a right hand coordinate system.\n")
        o.write("\n")
        o.write("   sky < 0, 1, 0 >\n")
        o.write("   right < %d, 0, 0>\n" % width)
        o.write("   up < 0, %d, 0 >\n" % height)
        o.write("   look_at < %g, %g, %g >\n" % (height * 0.74, height * 0.5, width * 0.79))
        o.write("}\n")
        o.write("\n")
        o.write("background { color rgb <0.0,0.0,0.0> }\n")
        o.write("\n")
        o.write("light_source { <-%g,-%g,%g>  rgb <0.6,0.6,0.6> }\n" % (height * 0.5136, height * 0.5136, width * 0.87))
        o.write("\n")
        o.write("\n")
        o.write("// macros for common shapes\n")
        o.write("\n")
        o.write("#default { finish {\n")
        o.write("   ambient 0.45\n")
        o.write("   diffuse 0.84\n")
        o.write("   specular 0.22\n")
        o.write("   roughness .00001\n")
        o.write("   metallic\n")
        o.write("   phong 0.9\n")
        o.write("   phong_size 120\n")
        o.write("}}\n")
        o.write("\n")
        o.write("#macro check_shadow()\n")
        o.write("   #if (noShadows)\n")
        o.write("       no_shadow \n")
        o.write("   #end\n")
        o.write("#end\n")
        o.write("\n")
        o.write("#declare slabZ = 0;\n")
        o.write("#declare depthZ = 2147483647;\n")
        o.write("#declare dzSlab = 10;\n")
        o.write("#declare dzDepth = dzSlab;\n")
        o.write("#declare dzStep = 0.001;\n")
        o.write("\n")
        o.write("#macro clip()\n")
        o.write("   clipped_by { box {<0,0,slabZ>,<Width,Height,depthZ>} }\n")
        o.write("#end\n")
        o.write("\n")
        o.write("#macro circleCap(Z,RADIUS,R,G,B,T)\n")
        o.write("   // cap for lower clip\n")
        o.write("   #local cutDiff = Z - slabZ;\n")
        o.write("   #local cutRadius2 = (RADIUS*RADIUS) - (cutDiff*cutDiff);\n")
        o.write("   #if (cutRadius2 > 0)\n")
        o.write("       #local cutRadius = sqrt(cutRadius2);\n")
        o.write("       #if (dzSlab > 0)\n")
        o.write("           #declare dzSlab = dzSlab - dzStep;\n")
        o.write("       #end\n")
        o.write("       cylinder{<X,Y,slabZ-dzSlab>,<X,Y,(slabZ+1)>,cutRadius\n")
        o.write("       pigment{rgbt<R,G,B,T>}\n")
        o.write("       translucentFinish(T)\n")
        o.write("       check_shadow()}\n")
        o.write("   #end\n")
        o.write("   // cap for upper clip\n")
        o.write("   #declare cutDiff = Z - depthZ;\n")
        o.write("   #declare cutRadius2 = (RADIUS*RADIUS) - (cutDiff*cutDiff);\n")
        o.write("   #if (cutRadius2 > 0)\n")
        o.write("       #local cutRadius = sqrt(cutRadius2);\n")
        o.write("       #if (dzDepth > 0)\n")
        o.write("           #declare dzDepth = dzDepth - dzStep;\n")
        o.write("       #end\n")
        o.write("       cylinder{<X,Y,depthZ+dzDepth>,<X,Y,(depthZ-1)>,cutRadius\n")
        o.write("       pigment{rgbt<R,G,B,T>}\n")
        o.write("       translucentFinish(T)\n")
        o.write("       check_shadow()}\n")
        o.write("   #end\n")
        o.write("#end\n")
        o.write("\n")
        o.write("#macro translucentFinish(T)\n")
        o.write("   #local shineFactor = T;\n")
        o.write("   #if (T <= 0.25)\n")
        o.write("       #declare shineFactor = (1.0-4*T);\n")
        o.write("   #end\n")
        o.write("   #if (T > 0.25)\n")
        o.write("       #declare shineFactor = 0;\n")
        o.write("   #end\n")
        o.write("   finish {\n")
        o.write("       ambient 0.45\n")
        o.write("       diffuse 0.84\n")
        o.write("       specular 0.22\n")
        o.write("       roughness .00001\n")
        o.write("       metallic shineFactor\n")
        o.write("       phong 0.9*shineFactor\n")
        o.write("       phong_size 120*shineFactor}\n")
        o.write("#end\n")
        o.write("\n")
        o.write("// atom\n")
        o.write("#macro a(X,Y,Z,RADIUS,R,G,B,T)\n")
        o.write("   sphere{<X,Y,Z>,RADIUS\n")
        o.write("       pigment{ rgbt<R,G,B,T> }\n")
        o.write("       translucentFinish(T)\n")
        o.write("       clip()\n")
        o.write("       check_shadow()}\n")
        o.write("#end\n")
        o.write("\n")
        o.write("// atom + ID\n")
        o.write("#macro at(X,Y,Z,RADIUS,R,G,B,T,SID,SV)\n")
        o.write("   sphere{<X,Y,Z>,RADIUS\n")
        o.write("     texture {\n")
        o.write("       pigment{ object { object{txt_id_v(X,Y,Z-RADIUS,SID,SV)} color rgbt<R,G,B,T> color White } }\n")
        o.write("       translucentFinish(T)\n")
        o.write("     }\n")
        o.write("   clip()\n")
        o.write("   finish { specular .5 }\n")
        o.write("   check_shadow()\n")
        o.write("   }\n")
        o.write("#end\n")
        o.write("\n")
        o.write("// bond\n")
        o.write("#macro b(X1,Y1,Z1,RADIUS1,X2,Y2,Z2,RADIUS2,R,G,B,T)\n")
        o.write("   cone{<X1,Y1,Z1>,RADIUS1,<X2,Y2,Z2>,RADIUS2\n")
        o.write("       pigment{ rgbt<R,G,B,T> }\n")
        o.write("       translucentFinish(T)\n")
        o.write("       clip()\n")
        o.write("       check_shadow()}\n")
        o.write("#end\n")
        o.write("\n")
        o.write("// open bond\n")
        o.write("#macro c(X1,Y1,Z1,RADIUS1,X2,Y2,Z2,RADIUS2,R,G,B,T)\n")
        o.write("   cone{<X1,Y1,Z1>,RADIUS1,<X2,Y2,Z2>,RADIUS2 open\n")
        o.write("       pigment{ rgbt<R,G,B,T> }\n")
        o.write("       translucentFinish(T)\n")
        o.write("       clip()\n")
        o.write("       check_shadow()}\n")
        o.write("#end\n")
        o.write("\n")
        o.write("// arrow\n")
        o.write("#macro arrow(X1,Y1,Z1,RADIUS1,X2,Y2,Z2,RADIUS2,R,G,B,T)\n")
        o.write("   cone{<X1,Y1,Z1>,RADIUS1,<X2,Y2,Z2>,RADIUS2\n")
        o.write("       pigment{ Green }\n")
        o.write("       translucentFinish(T)\n")
        o.write("       clip()\n")
        o.write("       check_shadow()}\n")
        o.write("#end\n")
        o.write("\n")
        o.write("// text\n")
        o.write("#macro txt_id_v(X,Y,Z,SID,SV)\n")
        o.write("  union {\n")
        o.write("   text {\n")
        o.write("       ttf \"cyrvetic.ttf\" SID 1,0\n")
        o.write("       scale <20,20,20>\n")
        o.write("       translate <X-10,Y+10,Z>\n")
        o.write("       pigment{ White }\n")
        o.write("       finish { reflection .25 specular 1 }\n")
        o.write("   }\n")
        o.write("   text {\n")
        o.write("       ttf \"cyrvetic.ttf\" SV 1,0\n")
        o.write("       scale <20,20,20>\n")
        o.write("       translate <X-26,Y-16,Z>\n")
        o.write("       pigment{ White }\n")
        o.write("       finish { reflection .25 specular 1 }\n")
        o.write("   }\n")
        o.write("  }\n")
        o.write("#end\n")
        o.write("\n")
        o.write("#macro tt(X,Y,Z,TXT)\n")
        o.write("   text {\n")
        o.write("       ttf \"cyrvetic.ttf\" TXT 1,0\n")
        o.write("       pigment{ White }\n")
        o.write("       finish { reflection .25 specular 1 }\n")
        o.write("       scale <20,20,20>\n")
        o.write("       translate <X-26,Y-15,Z>\n")
        o.write("  }\n")
        o.write("#end\n")
    return


# color table

CR = [1, 0.2, 0, 1, 1, 0, 1, 0.75, 0,      0, 0.75, 0.75,    0, 0.57, 0.5, 0,     0, 0.5, 0.5,   0, 0.5]
CG = [1, 0,   1, 0, 0.5, 1, 0, 0.75, 0,   0.75,    0, 0.75, 0.75,    0, 0.5, 0,   0.5,   0, 0.5, 0.5,   0]
CB = [1, 1,  0.1, 0, 0, 1, 1, 0.57, 0.75,   0,    0,    0, 0.75, 0.75, 0.5, 0.5,   0,   0,   0, 0.5, 0.5]

def write_mols(outfstub, VH_or_VL):

    [s1,s2] = find_scale_factor()
    # print(s1, s2)

    seg_beg = 0
    for m in range(0, root_nmols):
        V = []

        fnm = "%s%02d.dat" % (VH_or_VL, m+1)
        with open(fnm, "r") as f:
            lines = f.readlines()
            nlen = int(lines[0])
            for i in range(0,nlen):
                line = lines[i+1]
                items = line.split()
                V.append(float(items[0]))

        if m < 10:
            fnm = outfstub + "-0" + str(m + 1) + "-" + VH_or_VL + ".pov"
        else:
            fnm = outfstub + "-" + str(m + 1) + "-" + VH_or_VL + ".pov"

        print("Output POV-Ray file: %s" % (fnm))

        write_header(fnm)

        with open(fnm, "a") as o:
            o.write("\n")
            t0 = root_types[m]
            x0 = root_x[m] + height * 0.75
            y0 = root_y[m] + height * 0.5
            z0 = root_z[m] + width
            r = 40.0
            T = 0.0

            o.write("union {\n")

            # draw bonds
            for j in range(0, root_nneis[m]):
                k = seg_beg + j
                t = nei_types[k]
                x = nei_dx[k] * s1 + x0
                y = nei_dy[k] * s1 + y0
                z = nei_dz[k] * s2 + z0
                rr = nei_dr[k]
                k += 1
                o.write("b(%g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g, %g)\n" % (x0, y0, z0, r/8.0, x, y, z, r/8.0, CR[t], CG[t], CB[t], T))

            # draw central molecule
            o.write("\n")
            o.write("a(%g, %g, %g, %g, %g, %g, %g, %g)\n" % (x0, y0, z0, r, CR[t0], CG[t0], CB[t0], T))

            # draw neighbor molecules
            for j in range(0, root_nneis[m]):
                k = seg_beg + j
                t = nei_types[k]
                x = nei_dx[k] * s1 + x0
                y = nei_dy[k] * s1 + y0
                z = nei_dz[k] * s2 + z0
                rr = nei_dr[k]

                s_id=str(j+1)
                s_v = "%.2f" % V[j]
                o.write("at(%g, %g, %g, %g, %g, %g, %g, %g, \"%s\", \"%s\")\n" % (x, y, z, r, CR[t], CG[t], CB[t], T, s_id, s_v))

            o.write("}\n")

            # draw axles
            x1 = 120
            y1 = 100
            z1 = 100
            r1 = 6
            r2 = 6
            x2 = x1 + vec_a[0] * s1
            y2 = y1 + vec_a[1] * s1
            z2 = z1 + vec_a[2] * s2

            o.write("\n")

            o.write("union {\n")

            o.write("arrow(%g, Height-%g, %g, %g, %g, Height-%g, %g, %g, %g, %g, %g, %g)\n" % 
                    (x1, y1, z1, r1, x2, y2, z2, r2, 0.0, 1.0, 0.0, 0.0))
            o.write("arrow(%g, Height-%g, %g, %g, %g, Height-%g, %g, %g, %g, %g, %g, %g)\n" % 
                    (x2, y2, z2, r1*2.0, x2+r1*4.0, y2, z2, 0, 0.0, 1.0, 0.0, 0.0))
            o.write("tt(220, Height-80, 0, \"X\")\n")

            x2 = x1 + vec_b[0] * s1
            y2 = y1 + vec_b[1] * s1
            z2 = z1 + vec_b[2] * s2
            o.write("arrow(%g, Height-%g, %g, %g, %g, Height-%g, %g, %g, %g, %g, %g, %g)\n" % 
                    (x1, y1, z1, r1, x2, y2, z2, r2, 0.0, 1.0, 0.0, 0.0))
            o.write("arrow(%g, Height-%g, %g, %g, %g, Height-%g, %g, %g, %g, %g, %g, %g)\n" % 
                    (x2, y2, z2, r1*2.0, x2, y2+r1*4.0, z2, 0, 0.0, 1.0, 0.0, 0.0))
            o.write("tt(160, Height-180, 0, \"Y\")\n")

            x2 = x1 + vec_c[0] * s1
            y2 = y1 + vec_c[1] * s1
            z2 = z1 + vec_c[2] * s2
            o.write("arrow(%g, Height-%g, %g, %g, %g, Height-%g, %g, %g, %g, %g, %g, %g)\n" % 
                    (x1, y1, z1, r1, x2, y2, z2, r2, 0.0, 1.0, 0.0, 0.0))
            o.write("arrow(%g, Height-%g, %g, %g, %g, Height-%g, %g, %g, %g, %g, %g, %g)\n" % 
                    (x2, y2, z2, r1*2.0, x2, y2, z2+r1*4.0, 0, 0.0, 1.0, 0.0, 0.0))
            o.write("tt(60, Height-120, 0, \"Z\")\n")

            o.write("}\n")
            o.write("\n")

        seg_beg += root_nneis[m]

    return

def write_povray_batch():
    with open("povray_batch.py", "w") as o:
        o.write("#!/usr/bin/env python\n")
        o.write("\n")
        o.write("import os\n")
        o.write("import shutil\n")
        o.write("import sys\n")
        o.write("\n")
        o.write("directory = './'\n")
        o.write("\n")
        o.write("for file in [f for f in os.listdir(directory) if f.endswith('.pov')]:\n")
        o.write("    cmd = 'povray -W2400 -H1600 %s' % (file)\n")
        o.write("    os.system(cmd)\n")
        o.write("\n")
    return


# Finally output *.pov

write_mols("mol", "VH")
write_mols("mol", "VL")

write_povray_batch()
os.system("chmod 755 povray_batch.py")
print("Output run file: run_povray_batch.py")

print("All done.")


